{% load staticfiles %}
{% load hijack_tags %}

<!DOCTYPE html>
<html lang="en" ng-app="orchestra">
  <head>
    <base href="/orchestra/app/">

    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="">
    <meta name="author" content="Dashboard">
    <meta name="keyword" content="Dashboard, Bootstrap, Admin, Template, Theme, Responsive, Fluid, Retina">

    <!-- Favicon and Android phone icon -->
    <link rel="icon" type="image/png" href="{% static 'orchestra/icons/favicon.ico' %}">

    <!-- iPhone/iPad icon -->
    <link rel="apple-touch-icon" href="{% static 'orchestra/icons/apple-touch-icon.png' %}">

    <title data-ng-bind="title">{% block title %}{% endblock %} | Orchestra</title>

    <link rel="stylesheet" href="//maxcdn.bootstrapcdn.com/bootstrap/3.2.0/css/bootstrap.min.css" />
    <link rel="stylesheet" type="text/css" href="{% static 'orchestra/common/css/lib/style.css' %}">
    <link rel="stylesheet" type="text/css" href="{% static 'orchestra/common/css/lib/style-responsive.css' %}">

    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/font-awesome/4.6.1/css/font-awesome.min.css">

    <link rel="stylesheet" type="text/css" href="{% static 'hijack/hijack-styles.css' %}" />
    <link rel="stylesheet" type="text/css" href="{% static 'dist/orchestra/common/css/orchestra.css' %}">
    <link rel="stylesheet" type="text/css" href="//cdnjs.cloudflare.com/ajax/libs/angular-ui-select/0.16.1/select.min.css">

    {% block page_stylesheets %}
    {% endblock page_stylesheets %}

    {% block head %}
    {% endblock %}
    <!-- HTML5 shim and Respond.js IE8 support of HTML5 elements and media queries -->
    <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
      <script src="https://oss.maxcdn.com/libs/respond.js/1.4.2/respond.min.js"></script>
    <![endif]-->
  </head>

  <body>
  {{ request|hijackNotification }}
  <section id="container" >
    <!--header start-->
    <header class="header">
        <!--logo start-->
        <a href="/orchestra/app/" class="logo">
          <i class="fa fa-random"></i>
          <b>Orchestra</b>
        </a>
        <!--logo end-->
        {% block topbar %}
          <div class="nav notify-row" id="top_menu">
            <!--  notification start -->
            <ul class="nav top-menu">
            </ul>
          </div>
          <!--  notification end -->

          <div id="nav-options" class="dropdown">
            <div class="dropdown-toggle" id="dropdown-menu-main" data-toggle="dropdown" aria-haspopup="true" aria-expanded="true">
              {{ user.username }}
              <span class="caret"></span>
            </div>
            <ul class="dropdown-menu dropdown-menu-right" aria-labelledby="dropdown-menu-main">
              <li>
                <a href="/">
                  <i class="fa fa-dashboard"></i>
                  <span>Dashboard</span>
                </a>
              </li>

              <li>
                <a href="/orchestra/app/timecard/">
                  <i class="fa fa-clock-o"></i>
                  <span>Timecard</span>
                </a>
              </li>

              <li>
                <a href="/orchestra/communication/available_staffing_requests/">
                  <i class="fa fa-briefcase" aria-hidden="true"></i>
                  <span>Available Tasks</span>
                </a>
              </li>

              {% if is_project_admin %}
              <li>
                <a href="/orchestra/app/project/">
                  <i class="fa fa-tasks"></i>
                  <span>Project Management</span>
                </a>
              </li>
              {% endif %}

              <li>
                <a href="/orchestra/accounts/settings/">
                  <i class="fa fa-cog" aria-hidden="true"></i>
                  <span>Account Settings</span>
                </a>
              </li>

              <li>
                <a href="{% url 'logout_then_login' %}" target="_self">
                  <i class="fa fa-sign-out"></i>
                  <span>Sign out</span>
                </a>
              </li>
            </ul>
          </div>

          <work-timer class="work-timer"></work-timer>

        {% endblock %}
        <div class="top-menu">
        </div>
    </header>
    <!--header end-->

    <!-- MAIN SIDEBAR MENU -->
    {% block sidebar %}
    {% endblock %}

    <!-- MAIN CONTENT -->
    {% block main %}
    {% endblock %}

    {% block static_url %}
      <script type="text/javascript">
        // This is a noop function that is replaced by django static file
        // processing, but allows us to work in dev.
        function $static(url) {
          return url;
        }
      </script>
    {% endblock %}

    <!-- js placed at the end of the document so the pages load faster -->
    <script src="{% static "orchestra/common/js/lib/jquery.js" %}"></script>
    <script src="{% static "orchestra/common/js/lib/bootstrap.min.js" %}"></script>
    <script src="{% static "orchestra/common/js/lib/jquery-ui-1.9.2.custom.min.js" %}"></script>
    <script src="{% static "orchestra/common/js/lib/jquery.ui.touch-punch.min.js" %}"></script>
    <script class="include" type="text/javascript" src="{% static "orchestra/common/js/lib/jquery.dcjqaccordion.2.7.js" %}"></script>

    <!--script for this page-->
    {% block page_javascript %}
    {% endblock %}

    {% block third_party_scripts %}
      {% with template_name=ORCHESTRA_THIRD_PARTY_SCRIPTS_TEMPLATE %}
        {% if template_name %}
          {% include template_name %}
        {% endif %}
      {% endwith %}
   {% endblock %}

  </body>
</html>
