# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-27 14:57
from __future__ import unicode_literals

from django.db import migrations
import phonenumber_field.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0046_auto_20160426_2034'),
    ]

    operations = [
        migrations.AddField(
            model_name='worker',
            name='phone',
            field=phonenumber_field.modelfields.PhoneNumberField(
                max_length=128, null=True),
        ),
    ]
