# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-29 11:27
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0056_create_new_task_communication_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='staffingrequest',
            name='communication_preference',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to='orchestra.CommunicationPreference'),
        ),
    ]
