/**
 * Common stylesheet for Orchestra.
 *
 * This partial provides common variables and mixins useful for various view and
 * component styles.
 */

$main-blue: rgb(0, 121, 191);
$light-gray: rgb(204, 204, 204);
$dark-gray: rgb(60, 60, 60);
$light-green: rgb(46, 204, 113);
$dark-green: rgb(39, 174, 96);
$bold-yellow: rgb(230, 198, 55);
$black: rgb(0, 0, 0);

$sqrt-2: 1.4142;

$base-font-size: 10px;
$panel-margin: 3rem;
$panel-padding: 1.5rem;
$panel-button-margin: 1.5rem;

$main-font-stack: 'Ruda', sans-serif;

@mixin vendor-prefix($name, $argument) {
  -webkit-#{$name}: #{$argument};
  -ms-#{$name}: #{$argument};
  -moz-#{$name}: #{$argument};
  -o-#{$name}: #{$argument};
  #{$name}: #{$argument};
}

.disable-select {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

%no-padding {
  padding-left: 0;
  padding-right: 0;
}

.row.no-padding {
  @extend %no-padding;
  > [class *= "col-"] {
    @extend %no-padding;
  }
}

%loading-spinner {
  &::after {
    position: absolute;
    top: 0;
    left: 0;
    right: 0;
    bottom: 0;
    border-bottom: 1px solid $black;
    border-right: 1px solid $black;
    border-radius: 50%;

    content: "";
    animation: spin .75s linear infinite;
  }
}
