# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-19 20:39
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0040_timeentry_created_at'),
    ]

    operations = [
        migrations.AlterField(
            model_name='payrate',
            name='end_datetime',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='payrate',
            name='start_datetime',
            field=models.DateField(default=django.utils.timezone.now),
        ),
        migrations.RenameField(
            model_name='payrate',
            old_name='end_datetime',
            new_name='end_date',
        ),
        migrations.RenameField(
            model_name='payrate',
            old_name='start_datetime',
            new_name='start_date',
        ),

    ]
