# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-11-16 20:32
from __future__ import unicode_literals

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0067_staffbotrequest_last_inquiry_sent'),
    ]

    operations = [
        migrations.AddField(
            model_name='step',
            name='creation_policy',
            field=jsonfield.fields.JSONField(default={}),
        ),
    ]
