{% load i18n %}
<div class="account-tabs">
  <a href="{% url 'orchestra:account_settings' %}" class="btn btn-primary {{ account_settings }}">
    {% trans 'Account Settings' %}
  </a>
  <a href="{% url 'orchestra:communication_preference_settings' %}" class="btn btn-primary {{ communication_preference_settings }}">
    {% trans 'Notifications' %}
  </a>
  <a href="{% url 'auth_password_change' %}" class="btn btn-primary {{ password_change }}">
    {% trans 'Change Password' %}
  </a>
</div>
