# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-28 17:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0048_merge'),
    ]

    operations = [
        migrations.AddField(
            model_name='staffingrequest',
            name='communication_method',
            field=models.IntegerField(
                choices=[(0, 'slack'), (1, 'email')], default=0),
        ),
        migrations.AddField(
            model_name='staffingrequest',
            name='status',
            field=models.IntegerField(
                choices=[(0, 'pending'), (1, 'sent')], default=0),
        ),
    ]
